module RightPublish
  class MsiRepo < Repo
    DEFAULT_MSI_DIR = 'msi/'
    REPO_KEY = :msi_repo
    REPO_OPTIONS = {:subdir=>DEFAULT_MSI_DIR, :prune=>true}

    GEMS_SUBDIRECTORY = 'gems'
    MSI_EXT = 'msi'

    def add(file_or_dir, target)
      msi_list = get_pkg_list(file_or_dir, MSI_EXT)

      # Copy MSI files to the repository
      destination = repo_config[:subdir]
      msi_list.each do |path|
        # RubyGems repos do not normally prune; it must be explicitly enabled.
        if repo_config[:prune]
          do_in_subdir(destination) { prune_all("#{pkg_parts(path)[:name]}-*.msi") }
        end

        install_file(path, destination)
      end

      # No index generation is necessary; MSI is not a distribution mechanism. The annotated
      # HTML will suffice to direct people to the proper download.
    end

    def annotate(options={})
      options[:subdir] ||= repo_config[:subdir]
      options[:filter] = ['*.msi']
      super(options)
    end

    protected

    def pkg_parts(name)
      result = {:name=>nil, :version=>nil}
      result[:name], result[:version] = /([A-Za-z0-9\-_]+)-([0-9\.]+)\.#{MSI_EXT}\Z/.match(name).captures
      result
    end
  end

  RightPublish::RepoManager.register_repo(MsiRepo)
end
